@@ -2,7 +2,7 @@
 # codeskyblue 2020/06/03
 #
 
-__all__ = ['SafeStreamSocket', 'PlistSocket']
+__all__ = ['SafeStreamSocket', 'PlistSocket', 'PlistSocketProxy']
 
 import logging
 import os
@@ -39,6 +39,7 @@ class SafeStreamSocket:
         """
         self._id = get_uniq_id()
         self._sock = None
+        self._dup_sock = None # keep original sock when switch_to_ssl
         self._name = None
 
         if isinstance(addr, socket.socket):
@@ -57,18 +58,17 @@ class SafeStreamSocket:
                 family = socket.AF_INET
             self._sock = socket.socket(family, socket.SOCK_STREAM)
             self._sock.connect(addr)
-        
-        self._sock_gclist = [self._sock]
 
-        def _cleanup(socks: typing.List[socket.socket]):
+        def _cleanup():
             _id = str(self.id)
             if self.name:
                 _id = self.name + ":" + str(self.id)
             logger.debug("CLOSE(%s)", _id)
-            for sock in socks:
-                sock.close()
+            self._sock.close()
+            if self._dup_sock:
+                self._dup_sock.close()
 
-        self._finalizer = weakref.finalize(self, _cleanup, self._sock_gclist)
+        self._finalizer = weakref.finalize(self, _cleanup)
     
     def close(self):
         self._finalizer()
@@ -105,15 +105,18 @@ class SafeStreamSocket:
         return buf
 
     def sendall(self, data: Union[bytes, bytearray]) -> int:
-        with set_socket_timeout(self._sock, 10):
-            return self._sock.sendall(data)
+        return self._sock.sendall(data)
+
+    def ssl_unwrap(self):
+        assert isinstance(self._sock, ssl.SSLSocket)
+        self._sock.close()
+        self._sock = self._dup_sock
+        self._dup_sock = None
 
     def switch_to_ssl(self, pemfile):
         """ wrap socket to SSLSocket """
         # logger.debug("Switch to ssl")
         assert os.path.isfile(pemfile)
-        self._dup_sock = self._sock.dup()
-        self._sock_gclist.append(self._dup_sock)
         
         # https://docs.python.org/zh-cn/3/library/ssl.html#ssl.SSLContext
         context = ssl.SSLContext(ssl.PROTOCOL_TLS)
@@ -122,10 +125,11 @@ class SafeStreamSocket:
         except ssl.SSLError:
             # ignore: no ciphers can be selected.
             pass
+        self._dup_sock = self._sock.dup()
+
         context.load_cert_chain(pemfile, keyfile=pemfile)
         context.check_hostname = False
         ssock = context.wrap_socket(self._sock, server_hostname="iphone.localhost")
-        
         self._sock = ssock
 
     def __enter__(self):
@@ -198,30 +202,54 @@ class PlistSocket(SafeStreamSocket):
             logger.debug("RECV(%d): %s", self.id, payload)
         return payload
 
-    def send_recv_packet(self, payload: dict) -> dict:
-        self.send_packet(payload)
-        return self.recv_packet()
 
+class PlistSocketProxy:
+    def __init__(self, psock: typing.Union[PlistSocket, "PlistSocketProxy"]):
+        if isinstance(psock, PlistSocketProxy):
+            self._psock = psock._psock
+            psock._finalizer.detach()
+        else:
+            assert isinstance(psock, PlistSocket)
+            self._psock = psock
 
-class PlistSocketProperty:
-    def __init__(self, psock: PlistSocket):
-        self._psock = psock
+        self._finalizer = weakref.finalize(self, self._psock.close)
         self.prepare()
-        self._finalizer = weakref.finalize(self, self.psock.close)
     
     @property
     def psock(self) -> PlistSocket:
         return self._psock
     
+    @property
+    def name(self) -> str:
+        return self.psock.name
+    
+    @name.setter
+    def name(self, new_name: str):
+        self.psock.name = new_name
+    
     def prepare(self):
         pass
 
+    def get_socket(self) -> socket.socket:
+        return self.psock.get_socket()
+
     def send_packet(self, payload: dict, message_type: int = 8):
         return self.psock.send_packet(payload, message_type)
     
     def recv_packet(self, header_size=None) -> dict:
         return self.psock.recv_packet(header_size)
     
+    def send_recv_packet(self, payload: dict, timeout: float = 10.0) -> dict:
+        with set_socket_timeout(self.psock.get_socket(), timeout):
+            self.send_packet(payload)
+            return self.recv_packet()
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, *args):
+        self.close()
+
     def close(self):
         self._finalizer()
     
