@@ -16,7 +16,7 @@ from typing import Iterator, List, Union
 from . import bplist
 from . import struct2 as ct
 from ._proto import *
-from ._safe_socket import PlistSocketProperty
+from ._safe_socket import PlistSocketProxy
 from ._utils import pathjoin
 from .exceptions import MuxError, MuxServiceError
 
@@ -38,7 +38,7 @@ FHeader = ct.Struct("FHeader",
 logger = logging.getLogger(PROGRAM_NAME)
 
 
-class Sync(PlistSocketProperty):
+class Sync(PlistSocketProxy):
     def prepare(self):
         self.__tag = -1
 
@@ -331,7 +331,7 @@ class Sync(PlistSocketProperty):
         if info.is_dir():
             raise MuxError("{} is a directory", path)
         if info.is_link():
-            path = info['LinkTarget']
+            path = info.st_linktarget
 
         with self._context_open(path, AFC.O_RDONLY) as fd:
             left_size = info.st_size
@@ -359,10 +359,20 @@ class Sync(PlistSocketProperty):
         if isinstance(dst, str):
             dst = pathlib.Path(dst)
 
-        if src.as_posix() == "/" or self.stat(src).is_dir():
+        try:
+            finfo = self.stat(src)    
+        except MuxError as e:
+            logger.warning("Stat %s error: %s", src, e)
+            return
+
+        if src.as_posix() == "/" or finfo.is_dir():
             dst.mkdir(exist_ok=True)
             for fname in self.listdir(src):
-                self.pull(src.joinpath(fname), dst.joinpath(fname))
+                self.pull(src.joinpath(fname), dst.joinpath(fname), remove=remove)
+            try:
+                self.rmdir(src)
+            except:
+                pass
         else:
             if dst.is_dir():
                 dst = dst.joinpath(src.name)
@@ -370,11 +380,8 @@ class Sync(PlistSocketProperty):
             with dst.open("wb") as f:
                 for chunk in self.iter_content(src):
                     f.write(chunk)
-        if remove:
-            try:
-                self.rmtree(src)
-            except:
-                pass
+            if remove:
+                self.remove(src)
 
     def pull_content(self, path: str) -> bytearray:
         buf = bytearray()
